%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%       AUXILIARY PREDICATES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%
%% Document Identifier Delimiter = [46, 73]    
%% The code list [46, 73] is equivalent to '.I'
%% Usually the identifier is a number

isDocIdentifierDelimiter(Word) :- Word == [46, 73].



%%%%%%%%%%%
%% Text Delimiter = [46, 73]    
%% The code list [46, 87] is equivalent to '.W'

isTextDelimiter(Word) :- Word == [46, 87].



%%%%%%%%%%%
%% read_word(Word,EOF), extracts a Word from the current input stream.
%% EOF is bounded to 'true' if the end of the file is reached. It skips 
%% all symbols which a considered word delimiters.
%% WordDelimiter = [32, 44, 59, 10]
%% Code 32= ' ', 44=',', 59=';', 10 = NewLine

read_word(Word,EOF) :-
 		get0(C),
		C \== -1 -> %% Code -1 = End of File		
			(member(C, [32, 44, 59, 10]) -> 
				read_word(Word,EOF);
				read_remainderWord(RemainderWord,EOF), Word = [C|RemainderWord]
			);
			Word = [], EOF=true.

read_remainderWord(Word,EOF) :- 
		get0(C),
		C \== -1 -> %% Code -1 = End of File		
			(\+(member(C, [32, 44, 59, 10])) -> 
				read_remainderWord(RemainderWord,EOF), Word = [C|RemainderWord];
				Word = [], EOF=false
			);
			Word = [], EOF=true.



%%%%%%%%%%%%%%
%%       HIGH ORDER PREDICATES
%%%%%%%%%%%%%%
% applyTo(P, [X1, ..., Xn]) : Builds and executes the atom P(X1, ..., Xn)
%
applyTo(P, Xs) :- Atom =.. [P|Xs], call(Atom).



%%%%%%%%%%%%%%
%%       STATISTICS 
%%%%%%%%%%%%%%
% This is how time and space are measured.

% stat_start(I), Initialization procedure. I is the total number of inferences 
%              (passes via the call and redo ports) since the system was started.
% stat_start(output)
stat_start(I) :- garbage_collect, %%%% Sicstus % nogc, 
	   statistics(runtime,_), statistics(inferences,I).

% stat_end(I), "I" must be the total number of inferences computed by stat_start
% stat_end(input)
stat_end(I) :- statistics(runtime,[_,MSec]), Time is MSec/1000,
          statistics(inferences,I2), Inf is I2 - I, 
          statistics(global_stack,[GBytes,_]), MbG is GBytes/1048576,
          statistics(local_stack,[LBytes,_]),  MbL is LBytes/1048576,
          write('Runtime: '), write(Time), write(' sec   '), 
          write('Num. of inferences: '), write(Inf), write('  '), nl,
          write('MEMORY USAGE: '), 
          write('(Global stack) '), write(MbG), write(' Mb   '), 
          write('(Local stack) '), write(MbL), write(' Mb   '), 
          nl, nl.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
